! Dipfilter.x - 2D/3D dip (velocity) filtering
!
! USAGE:
!
!	Dipfilter.x <in.H >out.H [pass=1] [v1= v2= v3= v4= | d1= d2= d3
!= d4= ]
! 
! PARAMETERS:
!
!	pass=1	Pass dips in band
!	    =0	Reject dips in band
!
!	angle=0 Define band in terms of velocities
!	     =1 Define band in terms of angles
!
!	dim=2	2D dip-filter
!          =3	3D dip-filter
!
!	taper=1 linear taper
!            =2 cosine taper
!
!	Velocity/Dip band defined by:
!
!	    v2_____v3
!	     /     \
!	    /       \
!	 __/         \___
!	  v1         v4
!
! KEYWORDS: velocity filter dip
!
! AUTHOR: James Rickett - Oct 97
!
!% end self-doc
program zyxabc
implicit none
real, allocatable, dimension (:,:,:) :: data
complex, allocatable, dimension (:,:,:) :: cdata
integer n1,n2, n1pad,n2pad, pad2, dim
integer getch, hetch, putch
call initpar() 
call doc('/home/kana/sep/prof/gem/hlx/James/Dipfilter.rs') 
if (0==hetch('n1','d',n1)) then
  call seperr('Could not obtain n1 from history')
end if
if (0/=putch('From history: n1','d',n1)) then
  call seperr('trouble writing n1 to history file')
end if
if (0==hetch('n2','d',n2)) then
  call seperr('Could not obtain n2 from history')
end if
if (0/=putch('From history: n2','d',n2)) then
  call seperr('trouble writing n2 to history file')
end if
if (0==hetch('n3','d',n3)) then
  call seperr('Could not obtain n3 from history')
end if
if (0/=putch('From history: n3','d',n3)) then
  call seperr('trouble writing n3 to history file')
end if
if (0==getch('dim','d',dim)) then
  dim=2
end if
if (0/=putch('From par: dim','d',dim)) then
  call seperr('trouble writing dim to history file')
end if
n1pad=pad2(n1)
n2pad=pad2(n2)
n3pad=pad2(n3)
allocate (data(n1,n2,n3))
allocate (cdata(n1pad,n2pad,n3pad))
call main (n1,n2,n3,n1pad,n2pad,n3pad,data,cdata,dim)
end program zyxabc 
subroutine main(n1,n2,n3,n1pad,n2pad,n3pad,data,cdata,dim)
implicit none
integer n1,n2,n3,n1pad,n2pad,n3pad
integer i1,i2,i3,pass,angle,dim,nfr,ifr,taper
real data(n1,n2,n3)
complex cdata(n1pad,n2pad,n3pad)
real d1,d2,d3,kx0,dkx,kx,ky0,dky,ky,w0,dw,w,vel,degtan,v
real ang1, ang2, ang3, ang4, v1,v2,v3,v4,factor,taperfn
integer getch, hetch, putch
if (0==hetch('d1','f',d1)) then
  call seperr('Could not obtain d1 from history')
end if
if (0/=putch('From history: d1','f',d1)) then
  call seperr('trouble writing d1 to history file')
end if
if (0==hetch('d2','f',d2)) then
  call seperr('Could not obtain d2 from history')
end if
if (0/=putch('From history: d2','f',d2)) then
  call seperr('trouble writing d2 to history file')
end if
if (0==hetch('d3','f',d3)) then
  d3=1.
end if
if (0/=putch('From history: d3','f',d3)) then
  call seperr('trouble writing d3 to history file')
end if
if (0==getch('v1','f',v1)) then
  v1=0.
end if
if (0/=putch('From par: v1','f',v1)) then
  call seperr('trouble writing v1 to history file')
end if
if (0==getch('v2','f',v2)) then
  v2=0.1
end if
if (0/=putch('From par: v2','f',v2)) then
  call seperr('trouble writing v2 to history file')
end if
if (0==getch('v3','f',v3)) then
  v3=99999.
end if
if (0/=putch('From par: v3','f',v3)) then
  call seperr('trouble writing v3 to history file')
end if
if (0==getch('v4','f',v4)) then
  v4=999999.
end if
if (0/=putch('From par: v4','f',v4)) then
  call seperr('trouble writing v4 to history file')
end if
if (0==getch('v','f',v)) then
  v=1.
end if
if (0/=putch('From par: v','f',v)) then
  call seperr('trouble writing v to history file')
end if
if (0==getch('ang1','f',ang1)) then
  ang1=-50.
end if
if (0/=putch('From par: ang1','f',ang1)) then
  call seperr('trouble writing ang1 to history file')
end if
if (0==getch('ang2','f',ang2)) then
  ang2=-45.
end if
if (0/=putch('From par: ang2','f',ang2)) then
  call seperr('trouble writing ang2 to history file')
end if
if (0==getch('ang3','f',ang3)) then
  ang3=45.
end if
if (0/=putch('From par: ang3','f',ang3)) then
  call seperr('trouble writing ang3 to history file')
end if
if (0==getch('ang4','f',ang4)) then
  ang4=50.
end if
if (0/=putch('From par: ang4','f',ang4)) then
  call seperr('trouble writing ang4 to history file')
end if
if (0==getch('pass','d',pass)) then
  pass=1
end if
if (0/=putch('From par: pass','d',pass)) then
  call seperr('trouble writing pass to history file')
end if
if (0==getch('angle','d',angle)) then
  angle=0
end if
if (0/=putch('From par: angle','d',angle)) then
  call seperr('trouble writing angle to history file')
end if
if (0==getch('taper','d',taper)) then
  taper=1
end if
if (0/=putch('From par: taper','d',taper)) then
  call seperr('trouble writing taper to history file')
end if
if (angle .eq. 1) then
  v1=degtan(ang1)*v
  v2=degtan(ang2)*v
  v3=degtan(ang3)*v
  v4=degtan(ang4)*v
end if
if ((v1>=v2) .or.(v2>=v3) .or.(v3>=v4)) then
  call seperr('Need v1 < v2 < v3 < v4')
end if
ifr=1
nfr=1
if (dim .eq. 2) then
  nfr=n3
  n3=1
  n3pad=1
end if
! Loop over panels
do while  (ifr <= nfr)
  ifr=ifr+1
! Read the data from stdin
  call sreed('in',data,n1*n2*n3*4)
! Pad to power of 2
  do i3=1,n3pad  
    do i2=1,n2pad  
      do i1=1,n1pad  
        if ((i1 <= n1)  .and. (i2 <= n2)  .and. (i3 <= n3)) then
          cdata(i1,i2,i3)=cmplx(data(i1,i2,i3),0)
        else
          cdata(i1,i2,i3)=cmplx(0,0)
        end if 
      end do 
    end do 
  end do 
! 3D FFT
  call ft3D1axis(0, 1.,n1pad,n2pad,n3pad,cdata)
  call ft3D2axis(0,-1.,n1pad,n2pad,n3pad,cdata)
  if (dim .eq. 3) then
    call ft3D3axis(0,-1.,n1pad,n2pad,n3pad,cdata)
  end if
  dw =1/(n1pad*d1)
  w0 =-((n1pad-1)/2)*dw
  dkx=1/(n2pad*d2)
  kx0=-((n2pad-1)/2)*dkx
  dky=1/(n3pad*d3)
  ky0=-((n3pad-1)/2)*dky
! Apply mute
  do i3=1, n3pad  
    ky=(i3-1)*dky+ky0
    do i2=1, n2pad  
      kx=(i2-1)*dkx+kx0
      do i1=1, n1pad  
        w =(i1-1)*dw +w0
        vel=w/(sqrt(kx**2+ky**2))
        factor=1.
        if ((vel>=v1)  .and. (vel<=v2)) then
          factor=1-taperfn(v1,v2,vel,taper)
        end if
        if ((vel>=v2)  .and. (vel<=v3)) then
          factor=0.
        end if
        if ((vel>=v3)  .and. (vel<=v4)) then
          factor=taperfn(v3,v4,vel,taper)
        end if
        if (pass  .eq.  1) then
          factor=1.-factor
        end if
        cdata(i1,i2,i3)=cdata(i1,i2,i3)*factor
      end do 
    end do 
  end do 
! Inverse 3D FFT
  call ft3D1axis(1, 1.,n1pad,n2pad,n3pad,cdata)
  call ft3D2axis(1,-1.,n1pad,n2pad,n3pad,cdata)
  if (dim .eq. 3) then
    call ft3D3axis(1,-1.,n1pad,n2pad,n3pad,cdata)
  end if
! Truncate to original size
  do i3=1,n3  
    do i2=1,n2  
      do i1=1,n1  
        data(i1,i2,i3)=real(cdata(i1,i2,i3))
      end do 
    end do 
  end do 
! Write to stdout
  call srite('out',data,n1*n2*n3*4)
end do 
return
end
real function degtan(theta)
implicit none
real theta,pi
pi=3.14159267
degtan=tan(pi*theta/180)
return
end
real function taperfn(start,end,value,type)
implicit none
real start,end,value,pi
integer type
pi=3.14159267
taperfn=(value-start)/(end-start)
if (type.eq.2) then
  taperfn=sin(taperfn*pi/2)
end if
return
end 
