# Dipfilter.x - 2D/3D dip (velocity) filtering
#
# USAGE:
#
#	Dipfilter.x <in.H >out.H [pass=1] [v1= v2= v3= v4= | d1= d2= d3= d4= ]
# 
# PARAMETERS:
#
#	pass=1	Pass dips in band
#	    =0	Reject dips in band
#
#	angle=0 Define band in terms of velocities
#	     =1 Define band in terms of angles
#
#	dim=2	2D dip-filter
#          =3	3D dip-filter
#
#	taper=1 linear taper
#            =2 cosine taper
#
#	Velocity/Dip band defined by:
#
#	    v2_____v3
#	     /     \
#	    /       \
#	 __/         \___
#	  v1         v4
#
# KEYWORDS: velocity filter dip
#
# AUTHOR: James Rickett - Oct 97
#
#% end self-doc

integer n1,n2, n1pad,n2pad, pad2, dim
from history:	integer n1,n2,n3
from par:	integer dim=2
n1pad=pad2(n1); n2pad=pad2(n2);n3pad=pad2(n3);

allocate: real data(n1,n2,n3)
allocate: complex cdata(n1pad,n2pad,n3pad)

subroutine main(n1,n2,n3,n1pad,n2pad,n3pad,data,cdata,dim)

integer n1,n2,n3,n1pad,n2pad,n3pad
integer i1,i2,i3,pass,angle,dim,nfr,ifr,taper

real data(n1,n2,n3)
complex cdata(n1pad,n2pad,n3pad)
real d1,d2,d3,kx0,dkx,kx,ky0,dky,ky,w0,dw,w,vel,degtan,v
real ang1, ang2, ang3, ang4, v1,v2,v3,v4,factor,taperfn

from history: real d1,d2,d3=1.
from par: real v1=0., v2=0.1, v3=99999., v4=999999., v=1.
from par: real ang1=-50., ang2=-45., ang3=45., ang4=50.
from par: integer pass=1, angle=0, taper=1

if (angle==1) {
	v1=degtan(ang1)*v
	v2=degtan(ang2)*v
	v3=degtan(ang3)*v
	v4=degtan(ang4)*v
}
if ((v1>=v2)||(v2>=v3)||(v3>=v4)) call seperr('Need v1 < v2 < v3 < v4')

ifr=1; nfr=1
if (dim==2) { nfr=n3; n3=1; n3pad=1}

# Loop over panels
while (ifr <= nfr) { ifr=ifr+1

	# Read the data from stdin
	call sreed('in',data,n1*n2*n3*4)

	# Pad to power of 2
	do i3=1,n3pad {
	do i2=1,n2pad {
	do i1=1,n1pad {
		if ((i1 <= n1) && (i2 <= n2) && (i3 <= n3)) {
			cdata(i1,i2,i3)=cmplx(data(i1,i2,i3),0)
		} else {
			cdata(i1,i2,i3)=cmplx(0,0)
		}
	}}}

	# 3D FFT
	call ft3D1axis(0, 1.,n1pad,n2pad,n3pad,cdata)
	call ft3D2axis(0,-1.,n1pad,n2pad,n3pad,cdata)
	if (dim==3) call ft3D3axis(0,-1.,n1pad,n2pad,n3pad,cdata)

	dw =1/(n1pad*d1); w0 =-((n1pad-1)/2)*dw
	dkx=1/(n2pad*d2); kx0=-((n2pad-1)/2)*dkx
	dky=1/(n3pad*d3); ky0=-((n3pad-1)/2)*dky

	# Apply mute
	do i3=1, n3pad { ky=(i3-1)*dky+ky0
	do i2=1, n2pad { kx=(i2-1)*dkx+kx0
	do i1=1, n1pad { w =(i1-1)*dw +w0
		vel=w/(sqrt(kx**2+ky**2)); factor=1.
		if ((vel>=v1) && (vel<=v2)) factor=1-taperfn(v1,v2,vel,taper)
		if ((vel>=v2) && (vel<=v3)) factor=0.
		if ((vel>=v3) && (vel<=v4)) factor=taperfn(v3,v4,vel,taper)
		if (pass == 1) factor=1.-factor
		cdata(i1,i2,i3)=cdata(i1,i2,i3)*factor
	}}}

	# Inverse 3D FFT
	call ft3D1axis(1, 1.,n1pad,n2pad,n3pad,cdata)
	call ft3D2axis(1,-1.,n1pad,n2pad,n3pad,cdata)
	if (dim==3) call ft3D3axis(1,-1.,n1pad,n2pad,n3pad,cdata)

	# Truncate to original size
	do i3=1,n3 { 
	do i2=1,n2 { 
	do i1=1,n1 { 
		data(i1,i2,i3)=real(cdata(i1,i2,i3))
	}}}

	# Write to stdout
	call srite('out',data,n1*n2*n3*4)
}
return;end

real function degtan(theta)
real theta,pi
pi=3.14159267
degtan=tan(pi*theta/180)
return;end

real function taperfn(start,end,value,type)
real start,end,value,pi
integer type
pi=3.14159267
taperfn=(value-start)/(end-start)
if (type.eq.2) {
	taperfn=sin(taperfn*pi/2)
}
return;end
